import unittest
import os
import moto

import spark_emr.bootstrap
from spark_emr.const import PYTHON_VERSION


class TestBootstrap(unittest.TestCase):

    REGION = "eu-central-1"
    PYTHON_VERSION = "python35"
    BUCKET = "foocket"

    def _create_bucket(self):
        import boto3
        client = boto3.client('s3', region_name='eu-central-1')
        client.create_bucket(Bucket=self.BUCKET)
        return client

    def test_python_action(self):
        with moto.mock_s3():
            client = self._create_bucket()
            path = spark_emr.util.S3Path("s3://%s/barky" % self.BUCKET)
            bm = spark_emr.bootstrap.BootstrapManager(
                self.REGION, self.PYTHON_VERSION, path)
            step = bm._python_action()

            expected = {
                "Path": "s3://%s/barky/install_python.sh" % self.BUCKET,
                "Args": []

            }
            self.assertEqual(step["ScriptBootstrapAction"], expected)
            self.assertEqual(
                "barky/install_python.sh",
                client.list_objects(Bucket=self.BUCKET)['Contents'][0]["Key"]
            )
            obj = client.get_object(
                Bucket=self.BUCKET,
                Key="barky/install_python.sh")

            script = spark_emr.bootstrap.INSTALL_PYTHON.format(
                pv=self.PYTHON_VERSION,
                pvs=PYTHON_VERSION[self.PYTHON_VERSION])
            self.assertEqual(obj["Body"].read().decode("utf-8"), script)

    def test_pypi_action(self):
        with moto.mock_s3():
            client = self._create_bucket()
            path = spark_emr.util.S3Path("s3://%s/barky" % self.BUCKET)
            bm = spark_emr.bootstrap.BootstrapManager(
                self.REGION, self.PYTHON_VERSION, path)

            mypkg = "pip+mypkg==1.0.2"

            step = bm._pypi_action(mypkg)

            expected = {
                "Path": "s3://%s/barky/install_pypi_pypackage.sh" %
                self.BUCKET, "Args": []}
            self.assertEqual(step["ScriptBootstrapAction"], expected)
            self.assertEqual(
                "barky/install_pypi_pypackage.sh",
                client.list_objects(Bucket=self.BUCKET)['Contents'][0]["Key"]
            )

            obj = client.get_object(Bucket=self.BUCKET,
                                    Key="barky/install_pypi_pypackage.sh")

            script = spark_emr.bootstrap.INSTALL_PYPI.format(
                index_url="https://pypi.org/simple",
                package=mypkg.split("+")[1])
            self.assertEqual(obj["Body"].read().decode("utf-8"), script)

    def test_create_pypackage(self):
        bm = spark_emr.bootstrap.BootstrapManager(
            self.REGION, self.PYTHON_VERSION, "")
        package = os.path.dirname(os.path.dirname(__file__))

        temp_file = bm._create_pypackage(package)

        self.assertTrue(temp_file.endswith('.whl'))

    def test_local_pypackage_action(self):
        with moto.mock_s3():
            client = self._create_bucket()
            path = spark_emr.util.S3Path("s3://%s/barky" % self.BUCKET)
            bm = spark_emr.bootstrap.BootstrapManager(
                self.REGION, self.PYTHON_VERSION, path)
            package = os.path.dirname(os.path.dirname(__file__))
            step = bm._local_pypackage_action(package)

            expected = {
                "Path": "s3://%s/barky/install_local_pypackage.sh" %
                self.BUCKET,
                "Args": []}
            self.assertEqual(step["ScriptBootstrapAction"],
                             expected)
            self.assertEqual(
                "barky/install_local_pypackage.sh",
                client.list_objects(Bucket=self.BUCKET)['Contents'][0]["Key"]
            )

            obj = client.get_object(Bucket=self.BUCKET,
                                    Key="barky/install_local_pypackage.sh")

            self.assertTrue("aws s3 cp" in obj["Body"].read().decode("utf-8"))



import time
import uuid
import loguru

import boto3
import spark_emr.bootstrap

from spark_optimizer.optimizer import calculate_spark_settings
from spark_emr.step import spark_step, spark_yarn_log_step
from spark_emr.util import S3Path
from spark_emr.const import NAMESPACE

log = loguru.logger


class EmrManager(object):
    def __init__(self, config):
        self._config = config

        # instance configuration
        self._core_size_in_gb = config["core"]["size_in_gb"]
        self._core_type = config["core"]["instance_type"]
        self._master_type = config["master"]["instance_type"]
        self._master_size_in_gb = config["master"]["size_in_gb"]
        self._instance_count = config["core"]["instance_count"]
        self._ssh_key = config["ssh_key"]
        self._subnet_id = config["subnet_id"]
        self._optimization = config["optimization"]
        self._emr_version = config["emr_version"]
        self._consistent = config["consistent"]
        self._python_version = config["python_version"]
        self._bootstrap_uri = self.get_bootstrap_uri(
            config["bootstrap_uri"])
        self._region = config["region"]
        self._service_role = config["service_role"]
        self._job_flow_role = config["job_flow_role"]

        self.emr = boto3.client("emr", region_name=self._region)
        self.s3mgr = spark_emr.util.S3Manager(self._region)

        apps = ["Ganglia", "Spark", "Zeppelin"]
        self._apps_on_emr = [{"Name": app} for app in apps]

        # read remaining configuration
        self._log_uri = S3Path(self._bootstrap_uri).file_append("log").uri

        self.bm = spark_emr.bootstrap.BootstrapManager(
            self._region,
            self._python_version,
            self._bootstrap_uri
        )
        log.info("Used <<emr_config:%s>>" % self.__dict__)

    def get_bootstrap_uri(self, uri):
        uid = (uuid.uuid4()).hex
        return S3Path(uri).file_append(uid)

    def _create_classification_config(self):
        ret = [{
            "Classification": "spark",
            "Properties": {
                "maximizeResourceAllocation": "true",
            }
        }, {
            "Classification":
                "yarn-env",
            "Configurations": [{
                "Classification": "export",
                "Properties": {
                    "PYSPARK_PYTHON": self._python_version,
                    "PYSPARK_PYTHON_DRIVER": self._python_version
                }
            }]
        }, {
            "Classification": "core-site",
            "Properties": {
                "fs.s3a.endpoint": "s3.%s.amazonaws.com" % self._region
            }
        }, {
            "Classification": "emrfs-site",
            "Properties": {
                "fs.s3.consistent": "true" if self._consistent else "false",
            }
        }]

        if self._optimization:
            try:
                props, _ = calculate_spark_settings(self._core_type,
                                                    self._instance_count)
                props["spark.dynamicAllocation.enabled"] = "false"
                settings = {
                    "Classification": "spark-defaults",
                    "Properties": props
                }
                ret.append(settings)

                for x in ret:
                    if x["Classification"] == "spark":
                        x["Properties"]["maximizeResourceAllocation"] = "false"

            except Exception as e:
                log.warn("Unable to calculate <<spark_settings:%s>>" % e)

        log.info("Using EMR <<classification_config:%s>>" % ret)
        return ret

    def _create_instance_config(self):
        master = {
            "Name": "Master Instance",
            "InstanceRole": "MASTER",
            "InstanceType": self._master_type,
            "InstanceCount": 1,
            "EbsConfiguration": {
                    "EbsBlockDeviceConfigs": [
                        {
                            "VolumeSpecification": {
                                "VolumeType": "gp2",
                                "SizeInGB": self._master_size_in_gb
                            },
                            "VolumesPerInstance": 1
                        },
                    ],
                "EbsOptimized": True
            }
        }
        core = {
            "Name": "Core Instance",
            "InstanceRole": "CORE",
            "InstanceType": self._core_type,
            "InstanceCount": self._instance_count,
            "EbsConfiguration": {
                    "EbsBlockDeviceConfigs": [
                        {
                            "VolumeSpecification": {
                                "VolumeType": "gp2",
                                "SizeInGB": self._core_size_in_gb
                            },
                            "VolumesPerInstance": 1
                        },
                    ],
                "EbsOptimized": True
            }
        }
        ret = {
            "InstanceGroups": [master, core],
            "Ec2SubnetId": self._subnet_id,
            "Ec2KeyName": self._ssh_key,
            "KeepJobFlowAliveWhenNoSteps": True,
        }
        return ret

    def _launch(self,
                name,
                step=[],
                tag={},
                bootstrap_action=None):
        cluster_tag = []

        # add always a namespace
        if "namespace" not in tag:
            cluster_tag.append(
                {"Key": "namespace", "Value": NAMESPACE})
        for k, v in tag.items():
            cluster_tag.append({"Key": k, "Value": v})

        job_flow_details = self.emr.run_job_flow(
            Name=name,
            Instances=self._create_instance_config(),
            BootstrapActions=bootstrap_action,
            ServiceRole=self._service_role,
            ReleaseLabel=self._emr_version,
            Configurations=self._create_classification_config(),
            Applications=self._apps_on_emr,
            Steps=step,
            JobFlowRole=self._job_flow_role,
            LogUri=self._log_uri,
            VisibleToAllUsers=True,
            Tags=cluster_tag
        )
        self.job_flow_id = job_flow_details.get("JobFlowId")
        return self.job_flow_id

    def start(
            self,
            name,
            cmdline,
            package,
            tag,
            poll,
            yarn_log):

        self.s3mgr.delete(self._bootstrap_uri.key, self._bootstrap_uri.bucket)

        bootstrap_action = self.bm.bootstrap(package)
        log.info(
            "Uploaded bootstrap <<bootstrap_uri:%s>>" % self._bootstrap_uri)
        cmd_step = spark_step(name,
                              cmdline,
                              self._python_version, "TERMINATE_JOB_FLOW")
        yarn_log_step = spark_yarn_log_step(self._bootstrap_uri, self._region)

        step = [cmd_step]

        if yarn_log:
            step.append(yarn_log_step)

        job_flow_id = self._launch(name, step, tag, bootstrap_action)

        log.info("EMR <<job_flow_id:%s>>" % job_flow_id)

        if poll:
            self.waiting()
            log.info("EMR <<job_flow_id_finished:%s>>" % job_flow_id)
        return job_flow_id

    def add_steps(self, steps):
        return self.emr.add_job_flow_steps(
            JobFlowId=self.job_flow_id, Steps=steps)

    def status(self):
        return status(self.job_flow_id)

    def stop(self):
        return stop([self.job_flow_id], self._region)

    def waiting(self, polling_interval=10):
        while True:
            keep_waiting = self._step_finished()
            if not keep_waiting:
                break
            time.sleep(polling_interval)

        self.stop()

    def _step_finished(self):
        JOB_STATES = ["TERMINATED", "COMPLETED", "FAILED"]
        all_steps = self.emr.list_steps(
            ClusterId=self.job_flow_id)["Steps"]
        completed = [
            step["Status"]["State"] in JOB_STATES for step in all_steps
        ]
        finished = sum(completed)
        log.info(
            "Total <<job_number:%s>> are <<finished:%s>> with "
            "<<job_flow_id:%s>>" %
            (len(completed), finished, self.job_flow_id))
        return not all(completed)


def status(job_flow_id, region):
    emr = boto3.client("emr", region_name=region)
    return emr.describe_cluster(ClusterId=job_flow_id)


def stop(job_flow_ids, region):
    emr = boto3.client("emr", region_name=region)
    # XXX first remove tags than stop
    for job_flow_id in job_flow_ids:
        emr.remove_tags(ResourceId=job_flow_id, TagKeys=["namespace"])
    return emr.terminate_job_flows(JobFlowIds=job_flow_ids)


def list(namespace, region):
    emr = boto3.client("emr", region_name=region)
    ret = []

    def _collect(page):
        for p in page:
            cluster = status(p["Id"], region)["Cluster"]
            tag = {x["Key"]: x["Value"] for x in cluster["Tags"]}
            if "namespace" in tag and tag["namespace"] == namespace:
                ret.append(cluster)

    page = emr.list_clusters()
    _collect(page["Clusters"])
    while page.get("Marker"):
        page = emr.list_clusters(Marker=page["Marker"])
        _collect(page["Clusters"])

    return ret



import spark_emr.util

from spark_emr.const import SH_YARN_LOG

YARN_LOG = """
#!/bin/bash
set -x

LOCAL_DIR=/home/hadoop/yarn_logs
mkdir $LOCAL_DIR

for row in $( yarn application -appStates ALL -list | awk '/application_/{print $1 "," $2}' )
do
    fields=($(echo $row | tr "," "\n"))
    yarn logs -applicationId ${fields[0]} > $LOCAL_DIR/yarn_${fields[0]}_${fields[1]}.log
done
if [ -z "$(ls -A $LOCAL_DIR)" ]; then
    echo "($LOCAL_DIR) is empty"
else
    aws s3 cp $LOCAL_DIR/*.log %s
fi
"""  # noqa


def spark_yarn_log_step(bootstrap_base, region):
    s3mgr = spark_emr.util.S3Manager(region)
    uri = bootstrap_base.file_append(SH_YARN_LOG)
    yarn_log_sh = YARN_LOG % bootstrap_base.folder_uri
    s3mgr.put_content(target_uri=uri, content=yarn_log_sh)

    script = "/home/hadoop/%s" % SH_YARN_LOG
    cmd = "aws s3 cp {src} {script} && chmod +x {script} && bash {script}"
    cmd = cmd.format(src=uri.uri, script=script)

    step = {
        "Name": "Yarn log to s3",
        "HadoopJarStep": {
            "Jar": "command-runner.jar",
            "Args": ["bash", "-c", "%s" % cmd],
        },
        "ActionOnFailure": "CONTINUE",
    }
    return step


def spark_step(name,
               command,
               python_version,
               action_on_failure):

    ACTION_ON_FAILURE = ["TERMINATE_JOB_FLOW", "TERMINATE_CLUSTER",
                         "CANCEL_AND_WAIT", "CONTINUE"]
    if action_on_failure not in ACTION_ON_FAILURE:
        raise Exception("Wrong <<ActionOnFailure:%s>> use <<working:%s>>" % (
            action_on_failure, "|".join(ACTION_ON_FAILURE)))

    configuration = {
        "Name": name,
        "HadoopJarStep": {
            "Jar": "command-runner.jar",
            "Args": [
                "spark-submit", "--deploy-mode", "cluster", "--master",
                "yarn",
                "--conf",
                "spark.yarn.appMasterEnv.PYSPARK_PYTHON=%s" % python_version,
                "--conf",
                "spark.executorEnv.PYSPARK_PYTHON=%s" % python_version,
            ],
        },
        "ActionOnFailure": action_on_failure,
    }

    args = [x.strip() for x in command.split()]
    args[0] = "/usr/local/bin/%s" % args[0]
    configuration["HadoopJarStep"]["Args"].extend(args)
    return configuration



import unittest
import os
import moto
import pytest
from spark_emr.__main__ import main

HERE = os.path.dirname(os.path.abspath(__file__))
CONFIG = os.path.join(HERE, "config.yaml")


class TestMain(unittest.TestCase):
    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    def test_cli(self):
        self.assertRaises(SystemExit, main, [])
        out, err = self.capsys.readouterr()
        self.assertTrue(
            "{start,status,list,stop}" in out)

    def test_cli_start(self):
        self.assertRaises(SystemExit, main, ["start"])
        out, err = self.capsys.readouterr()
        self.assertTrue(
            "required: --name, --cmdline, --package" in
            err)

    def test_cli_stop(self):
        self.assertRaises(SystemExit, main, ["stop"])
        out, err = self.capsys.readouterr()
        self.assertTrue("required: --cluster-id" in err)

    def test_cli_status(self):
        self.assertRaises(SystemExit, main, ["status"])
        out, err = self.capsys.readouterr()
        self.assertTrue("required: --cluster-id" in err)

    def test_cli_list(self):
        with moto.mock_emr():
            main(["list", "--config", CONFIG])
            out, err = self.capsys.readouterr()
            self.assertTrue("id, name, reason, state, created" in out)



import os
import loguru

from urllib.parse import urlparse

import yaml

import boto3

log = loguru.logger


class S3Path(object):
    def __init__(self, uri):
        if isinstance(uri, S3Path):
            uri = uri.uri
        self.uri = uri
        self.parsed = urlparse(uri)

    @property
    def folder_uri(self):
        if self.uri.endswith("/"):
            return self.uri
        return self.uri + "/"

    @property
    def path(self):
        return os.path.split(self.key)[0]

    @property
    def bucket(self):
        return self.parsed.netloc

    @property
    def key(self):
        return self.parsed.path.lstrip("/")

    @property
    def file(self):
        return os.path.basename(self.parsed.path)

    def file_append(self, name):
        new_url = os.path.join(self.parsed.geturl(), name)
        return S3Path(new_url)

    def __eq__(self, other):
        return self.uri == other.uri

    def __repr__(self):
        return "<<%s:%s>>" % (self.__class__.__name__, self.uri)


class S3Manager(object):

    def __init__(self, region):
        self.region = region
        self.s3c = boto3.client('s3', region_name=region)
        self.s3res = boto3.resource('s3', region_name=region)

    def delete(self, key, bucket):
        if self.s3res.Bucket(bucket) in self.s3res.buckets.all():
            self.s3c.delete_object(Bucket=bucket, Key=key)

    def delete_all(self, bucket):
        bucket = self.s3res.Bucket(bucket)
        bucket.objects.all().delete()

    def put_content(self, target_uri, content):
        self.s3c.put_object(
            Body=content, Bucket=target_uri.bucket, Key=target_uri.key)
        return target_uri

    def upload(self, local, target_uri):
        if self.s3res.Bucket(
                target_uri.bucket) not in self.s3res.buckets.all():
            self.s3res.create_bucket(
                Bucket=target_uri.bucket,
                CreateBucketConfiguration={'LocationConstraint': self.region})
        self.s3c.upload_file(local, target_uri.bucket, target_uri.key)
        return target_uri.uri


def load_config(path):
    if not os.path.exists(path):
        raise Exception("Config file not found!")
    config = yaml.load(open(path, 'r'))
    return config



import sys
import loguru
import argparse

import spark_emr.manager

from spark_emr.util import load_config
from spark_emr.const import CONFIG_DIR, NAMESPACE

log = loguru.logger


def base_parser(parser):
    parser.add_argument(
        "--config",
        help="the path of the config yaml default (%s)" % CONFIG_DIR,
        default=CONFIG_DIR)
    return parser


def base_parser_start(parser):
    parser = base_parser(parser)
    parser.add_argument("--name", help="Cluster name", required=True)
    parser.add_argument("--tags", help="Tags", nargs="*")
    parser.add_argument(
        "--cmdline", help="The command line to run the model", required=True)
    parser.add_argument(
        "--package",
        help="A package path or name on for pip",
        required=True)
    parser.add_argument("--poll", dest="poll", action="store_true")
    parser.add_argument("--no-poll", dest="poll", action="store_false")
    parser.set_defaults(poll=True)

    parser.add_argument("--yarn-log", dest="yarn_log", action="store_true")
    parser.add_argument("--no-yarn-log", dest="yarn_log", action="store_false")
    parser.set_defaults(yarn_log=False)


def get_func_list(parser):
    parser = base_parser(parser)
    parser.add_argument("--namespace",
                        help="Cluster namespace",
                        required=False,
                        default=NAMESPACE)
    return _list


def _list(param):
    config = load_config(param.config)
    print("id, name, reason, state, created")
    for x in spark_emr.manager.list(param.namespace, config["region"]):
        line = ", ".join(
            [x["Id"],
             x["Name"],
             x["Status"].get("StateChangeReason", {})["Code"],
             x["Status"]["State"],
             str(x["Status"]["Timeline"]["CreationDateTime"])])
        print(line)


def get_func_stop(parser):
    parser = base_parser(parser)
    parser.add_argument("--cluster-id", help="Cluster id", required=True)
    return _stop


def _stop(param):
    config = load_config(param.config)
    spark_emr.manager.stop([param.cluster_id], config["region"])


def get_func_status(parser):
    parser = base_parser(parser)
    parser.add_argument("--cluster-id", help="Cluster id", required=True)
    return _status


def _status(param):
    config = load_config(param.config)
    ret = spark_emr.manager.status(param.cluster_id, config["region"])
    print(ret)


def get_func_start(parser):
    parser = base_parser_start(parser)
    return _start


def _start(param):
    config = load_config(param.config)
    emr_mgr = spark_emr.manager.EmrManager(config)

    emr_mgr.start(
        param.name,
        param.cmdline,
        param.package,
        param.tags,
        param.poll,
        param.yarn_log
    )


def main(args=None):
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help="available commands", dest="mode")

    parser_start = subparsers.add_parser("start")
    parser_status = subparsers.add_parser("status")
    parser_list = subparsers.add_parser("list")
    parser_stop = subparsers.add_parser("stop")

    func_start = get_func_start(parser_start)
    func_status = get_func_status(parser_status)
    func_lists = get_func_list(parser_list)
    func_stop = get_func_stop(parser_stop)

    param = parser.parse_args(args)
    try:
        if param.mode == "status":
            func_status(param)
        elif param.mode == "list":
            func_lists(param)
        elif param.mode == "stop":
            func_stop(param)
        elif param.mode == "start":
            param.tags = dict(zip(param.tags[::2], param.tags[1::2]))
            func_start(param)
        else:
            parser.print_help()
            sys.exit(1)
    except Exception as e:
        print("ERROR: %s\n" % "".join(e.args))
        log.exception(e)
        parser_start.print_help()
        sys.exit(1)


if __name__ == "__main__":
    main()



import unittest

import spark_emr
import spark_emr.util


class TestS3Path(unittest.TestCase):

    def test_main(self):
        parse = spark_emr.util.S3Path(
            "s3://bucket/folder/file.txt")
        self.assertEqual(parse.key, "folder/file.txt")
        self.assertEqual(parse.file, "file.txt")
        self.assertEqual(parse.bucket, "bucket")
        self.assertEqual(parse.path, "folder")
        self.assertEqual(parse.uri, "s3://bucket/folder/file.txt")

    def test_file_append(self):
        parse = spark_emr.util.S3Path("s3://bucket/folder/")
        self.assertEqual(
            parse.file_append("some_file.sh").uri,
            "s3://bucket/folder/some_file.sh")
        parse = spark_emr.util.S3Path("s3://bucket/folder")
        self.assertEqual(
            parse.file_append("some_file.sh").uri,
            "s3://bucket/folder/some_file.sh")

    def test_compare(self):
        first = spark_emr.util.S3Path("s3://bucket/folder/")
        second = spark_emr.util.S3Path("s3://bucket/folder/")
        assert first == second
        # neq
        first = spark_emr.util.S3Path("s3://bucket/folder/")
        second = spark_emr.util.S3Path("s3://bucket/some-folder/")
        assert first != second



import sys

from random import random
from operator import add

from pyspark.sql import SparkSession


def main():
    spark = (SparkSession
             .builder
             .appName("pi")
             .getOrCreate())
    print("Spark: %s" % spark)

    partitions = int(sys.argv[1]) if len(sys.argv) > 1 else 2
    n = 100000 * partitions

    def f(_):
        x = random() * 2 - 1
        y = random() * 2 - 1
        return 1 if x ** 2 + y ** 2 <= 1 else 0

    count = spark.sparkContext.parallelize(
        range(1, n + 1), partitions).map(f).reduce(add)
    print("Pi is roughly %f" % (4.0 * count / n))

    spark.stop()



import unittest
import os
import moto
import spark_emr.util
import spark_emr.manager

HERE = os.path.dirname(os.path.abspath(__file__))
DUMMY_FOLDER = os.path.join(HERE, "dummy")
CONFIG = os.path.join(HERE, "config.yaml")


class TestEMRManager(unittest.TestCase):

    def _create_bucket(self, bucket):
        import boto3
        client = boto3.client('s3', region_name='eu-central-1')
        client.create_bucket(Bucket=bucket)
        return client

    def xxtest_start(self):
        # need a fix https://github.com/spulec/moto/issues/1708
        with moto.mock_s3():
            with moto.mock_emr():
                config = spark_emr.util.load_config(CONFIG)
                bootstrap_path = spark_emr.util.S3Path(config["bootstrap_uri"])
                self._create_bucket(bootstrap_path.bucket)
                emr = spark_emr.manager.EmrManager(config)
                job_flow_id = emr.start(
                    "TEST-EMR",
                    "spark_emr_dummy.py 10",
                    DUMMY_FOLDER,
                    {},
                    False,
                    False)
                self.assertIsNotNone(job_flow_id)

    def test_list(self):
        with moto.mock_s3():
            with moto.mock_emr():
                config = spark_emr.util.load_config(CONFIG)
                ret = spark_emr.manager.list("spark_emr", config["region"])
                self.assertEqual(ret, [])



import os

from setuptools import find_packages, setup

version = "0.1.1"

here = os.path.abspath(os.path.dirname(__file__))
CHANGES = open(os.path.join(here, "CHANGES.md")).read()
README = open(os.path.join(here, "README.md")).read()

setup(
    name="spark_emr",
    version=version,
    description="Run python packages on AWS EMR",
    long_description=README + "\n\n" + CHANGES,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    keywords="AWS EMR SPARK PYSPARK",
    author="Josip Delic",
    author_email="delijati@gmx.net",
    url="https://www.github.com/delijati/spark-emr",
    license="MIT",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        "boto3",
        "loguru",
        "pyyaml",
        "spark-optimizer",
        # we need this to get tox working
        "pluggy>=0.7",
    ],
    test_suite="test",
    entry_points={
        "console_scripts": [
            "spark-emr = spark_emr.__main__:main"
        ],
    }
)



import unittest

import moto

from spark_emr.step import spark_yarn_log_step, spark_step
from spark_emr.util import S3Path


class TestSparkStep(unittest.TestCase):
    def test_spark_step(self):
        name = 'etl'
        command = "etl.py --in in.parquet --out out.parquet"

        configuration = spark_step(name, command, "python35", "CONTINUE")

        self.assertTrue(isinstance(configuration, dict))
        self.assertEqual(name, configuration['Name'])
        self.assertEqual(configuration.get('ActionOnFailure'), 'CONTINUE')
        self.assertEqual(len(configuration['HadoopJarStep']['Args']), 14)
        self.assertTrue("/usr/local/bin/etl.py" in configuration[
            'HadoopJarStep']['Args'])

    def test_yarn_log_step(self):
        with moto.mock_s3():
            # needed for moto
            import boto3
            client = boto3.client('s3', region_name='eu-central-1')
            client.create_bucket(Bucket='bucket')

            base_uri = S3Path("s3://bucket/folder")
            step = spark_yarn_log_step(base_uri, "eu-central-1")

            self.assertTrue(isinstance(step, dict))
            self.assertEqual(step["HadoopJarStep"]["Args"], [
                'bash', '-c',
                'aws s3 cp s3://bucket/folder/yarn_log.sh /home/hadoop/yarn_log.sh && chmod +x /home/hadoop/yarn_log.sh && bash /home/hadoop/yarn_log.sh'
            ])  # noqa



import os

home = os.path.expanduser("~")

CONFIG_DIR = os.path.join(home, ".config", "spark-emr.yaml")
NAMESPACE = "spark_emr"

SH_INSTALL_PYTHON = "install_python.sh"
SH_YARN_LOG = "yarn_log.sh"
SH_PYPI_PYPACKAGE = "install_pypi_pypackage.sh"
SH_LOCAL_PYPACKAGE = "install_local_pypackage.sh"
PYTHON_VERSION = {"python35": "3.5", "python36": "3.6"}



import os
import uuid
import glob
import shutil
import loguru
import tempfile
import subprocess

import spark_emr.util
from spark_emr.const import (SH_INSTALL_PYTHON, SH_LOCAL_PYPACKAGE,
                             SH_PYPI_PYPACKAGE,
                             PYTHON_VERSION)

log = loguru.logger

INSTALL_PYTHON = """
#!/bin/bash
set -ex
sudo yum install -y {pv} {pv}-devel {pv}-setuptools
sudo easy_install-{pvs} pip
sudo /usr/local/bin/pip3 install --upgrade pip
"""

INSTALL_PYPI = """
#!/bin/bash
set -ex
sudo /usr/local/bin/pip3 install -i {index_url} {package}
"""  # noqa

INSTALL_PYPACKAGE = """
#!/bin/bash
set -ex
aws s3 cp {path} {name}.whl
sudo /usr/local/bin/pip3 install {name}.whl
"""


class BootstrapManager(object):

    def __init__(self, region, python_version, bootstrap_uri):
        self.region = region
        self.python_version = python_version
        self.bootstrap_base = bootstrap_uri
        self.s3mgr = spark_emr.util.S3Manager(region)

    def call_shell(self, cmd):
        subprocess.call(
            cmd,
            shell=True,
            stdout=open(
                os.devnull,
                "w"))

    def _create_pypackage(self, package):
        tempfolder = os.path.join(tempfile.gettempdir(),
                                  uuid.uuid4().hex)
        build_directory = os.path.join(package, "build")
        shutil.rmtree(build_directory, ignore_errors=True)

        # create the pypackage file which contains the source code
        pypackage_cmd = "cd %s && python3 setup.py bdist_wheel -d %s" % (
            package, tempfolder)
        self.call_shell(
            "mkdir {dir} && chmod 777 {dir}".format(
                dir=tempfolder))
        self.call_shell(pypackage_cmd)

        log.info("Created <<package:%s>> in <<pydir:%s>>" % (
            package, tempfolder))
        ret = glob.glob(os.path.join(tempfolder, "*.whl"))
        assert len(ret) == 1
        return ret[0]

    def _python_action(self):
        install_python = INSTALL_PYTHON.format(
            pv=self.python_version,
            pvs=PYTHON_VERSION[self.python_version])
        install_python_uri = self.bootstrap_base.file_append(SH_INSTALL_PYTHON)
        self.s3mgr.put_content(
            target_uri=install_python_uri,
            content=install_python)
        log.info("Uploaded to <<install_python_uri:%s>>" % install_python_uri)

        ret = {
            "Name": "Install python",
            "ScriptBootstrapAction": {
                "Path": install_python_uri.uri,
                "Args": []
            }
        }
        return ret

    def _pypi_action(
            self,
            package,
            index_url="https://pypi.org/simple"):
        package = package.split("pip+")[1]
        install_from_pypi = self.bootstrap_base.file_append(SH_PYPI_PYPACKAGE)
        install_pypi = INSTALL_PYPI.format(
            index_url=index_url, package=package)
        self.s3mgr.put_content(
            target_uri=install_from_pypi, content=install_pypi)
        log.info("Uploaded <<install_pypi_uri:%s>>" % install_from_pypi)

        ret = {
            "Name": "Install package from pypi",
            "ScriptBootstrapAction": {
                "Path": install_from_pypi.uri,
                "Args": []
            }
        }
        return ret

    def _local_pypackage_action(self, package):
        # create new pypackage
        pypackage_path = self._create_pypackage(package)
        pypackage_name = pypackage_path.split("/")[-1]
        # upload to S3
        pypackage_uri = self.bootstrap_base.file_append(pypackage_name)
        self.s3mgr.upload(pypackage_path, pypackage_uri)
        log.info("Uploaded <<local_papackage_uri:%s>>" % pypackage_uri)

        # push script for installing the local pypackage to S3
        local_uri = self.bootstrap_base.file_append(SH_LOCAL_PYPACKAGE)
        install_pypackage = INSTALL_PYPACKAGE.format(
            name=pypackage_name, path=pypackage_uri.uri)
        self.s3mgr.put_content(
            target_uri=local_uri, content=install_pypackage)
        log.info("Uploaded <<install_local_uri:%s>>" % local_uri.uri)

        ret = {
            "Name": "Install package from pypi",
            "ScriptBootstrapAction": {
                "Path": local_uri.uri,
                "Args": []
            }
        }
        return ret

    def bootstrap(self, package):
        python_action = self._python_action()

        if package.startswith("pip+"):
            pkg_action = self._pypi_action(package)
        else:
            pkg_action = self._local_pypackage_action(package)

        return [python_action, pkg_action]



from setuptools import setup


setup(
    name='spark_emr_dummy',
    version="0.0.1",
    py_modules=['main'],
    entry_points={
        'console_scripts': [
            'spark_emr_dummy.py = main:main'
        ],
    }
)
